<?php
/**
 * Datasource Syntax Plugin.
 *
 * @license  Apache 2 (http://www.apache.org/licenses/LICENSE-2.0)
 * @author   Kazuyuki Matsuda / kazm.atmark.shorindo.com
 */
// must be run within Dokuwiki
if (!defined('DOKU_INC')) die();

if (!defined('DOKU_LF')) define('DOKU_LF', "\n");
if (!defined('DOKU_TAB')) define('DOKU_TAB', "\t");
if (!defined('DOKU_PLUGIN')) define('DOKU_PLUGIN',DOKU_INC.'lib/plugins/');
require_once(DOKU_PLUGIN.'syntax.php');

class syntax_plugin_datasource extends DokuWiki_Syntax_Plugin {

	function getInfo() {
		return confToHash(dirname(__FILE__).'/plugin.info.txt');
	}

	function getType() { return 'container'; }
	function getSort() { return 500; }
	function getPType() { return 'block';}

	function connectTo($mode) {
		$this->Lexer->addSpecialPattern(
                    '<datasource[^>]*?[^/]>.*?</datasource>',
                    $mode, 'plugin_datasource');
		$this->Lexer->addSpecialPattern(
                    '<datasource[^>]*?/>',
                    $mode, 'plugin_datasource');
	}

	function handle($match, $state, $pos, &$handler) {
		$params = array();
		if (preg_match('/<datasource([^>]*?[^\/])>(.*?)<\/datasource>/s', $match, $folder)) {
			$params = $this->read_attr($folder[1]);
			$params['data'] = preg_replace('/^\s+/s', '', $folder[2]);
		} else if (preg_match('/<datasource([^>]*?)\/>/s', $match, $folder)) {
			$params = $this->read_attr($folder[1]);
		}
		return array($state, $params);
	}

	function read_attr($str) {
		$attrs = array();
		if (preg_match_all("/(\S+?)=\"([^\"]*)\"/", $str, $m, PREG_SET_ORDER)) {
			foreach ($m as $p) {
				$attrs[$p[1]] = $p[2];
			}
		}
		return $attrs;
	}

	function render($mode, &$renderer, $data) {
		if($mode == 'xhtml'){
			list($state, $params) = $data;
            list($name, $opt) = explode("?", $params['name'], 2);
            $opts = array();
            foreach (explode("&", $opt) as $o) {
                if (preg_match("/^(.*?)=(.*)$/", $o, $m)) {
                    $opts[$m[1]] = $m[2];
                } else {
                    $opts[$o] = true;
                }
            }
            $dso = &plugin_load('helper', 'datasource');
            $dom = $dso->read($params['name']);
            $renderer->doc .= '<pre class="code">'.htmlspecialchars($dom->saveXML()).'</pre>';
			return true;
		}

		return false;
	}

}
